<?php

namespace Iwaydev\TransNgClient;

use \DomainException;
use Iwaydev\BaseSender\Sender;
use Iwaydev\TransNgClient\events\TransNgErrorResponseEvent;
use JsonException;
use Psr\Http\Message\ResponseInterface;
use Psr\Log\LoggerInterface;
use Psr\SimpleCache\CacheInterface;

/**
 * Клиент NG
 *
 * Class TransNgBackendClient
 *
 * @package Iwaydev\TransNgClient
 */
class TransNgBackendClient extends Sender
{
    /**
     * @param string $url URI запроса
     * @param string $version Версия используемого API
     * @param string $serviceUserLogin Логин сервисного пользователя
     * @param string $serviceUserPassword Пароль сервисного пользователя
     * @param CacheInterface|null $cache Инстанс кэшера
     * @param int $cacheLifetime Время жизни кэша в секундах
     * @param LoggerInterface|null $logger Инстанс логгера
     * @param string|null $xUuid UUID
     */
    public function __construct(
        string $url,
        string $version,
        private readonly string $serviceUserLogin,
        private readonly string $serviceUserPassword,
        ?CacheInterface $cache = null,
        int $cacheLifetime = 3600,
        ?LoggerInterface $logger = null,
        ?string $xUuid = null,
    ) {
        parent::__construct(
            $url,
            $version,
            $cache,
            $cacheLifetime,
            $logger,
            $xUuid
        );
    }

    /**
     * Авторизоваться под сервисным пользователем
     *
     * @return $this
     * @throws JsonException
     */
    public function authServiceUser(): self
    {
        $this->login($this->serviceUserLogin, $this->serviceUserPassword);

        return $this;
    }

    /**
     * Авторизация
     *
     * @param string $token Токен авторизации
     *
     * @return $this
     */
    public function auth(string $token): self
    {
        $this->setHeader('Authorization', 'Bearer ' . $token);

        return $this;
    }

    /**
     * Авторизация
     *
     * @param string $login
     * @param string $password
     *
     * @return self
     * @throws JsonException
     */
    public function login(string $login, string $password): self
    {
        $response = $this->post('auth/login', [
            'login' => $login,
            'password' => $password,
        ]);

        return $this->auth($response['token']);
    }

    /**
     * Возвращает поездки согласно переданным фильтрам
     *
     * @param array $query Фильтры
     *
     * @return array
     * @throws JsonException
     */
    public function getTrips(array $query = []): array
    {
        return $this->get('orders', $query);
    }

    /**
     * @param array $query
     *
     * @return array
     * @throws JsonException
     */
    public function getUsers(array $query = []): array
    {
        return $this->get('users', $query);
    }

    /**
     * @param array $query
     *
     * @return array
     * @throws JsonException
     */
    public function getCompanies(array $query = []): array
    {
        return $this->get('users/companies', $query);
    }

    /**
     * Метод для получения настроек пользователя
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function getSettingUser(array $data = []): array
    {
        return $this->put('partners/setting-user', $data);
    }

    /**
     * Метод отправки ваучера по заказу
     *
     * @param int $transaction
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function documentsSend(int $transaction, array $data = []): array
    {
        return $this->post('documents/send/' . $transaction, $data);
    }

    /**
     * Получение статистики по основным компонентам проекта
     *
     * @param array $data
     *
     * @return array
     * @throws JsonException
     */
    public function monitor(array $data = []): array
    {
        return $this->get('monitor', $data);
    }

    /**
     * Подготовка ответа
     *
     * @param ResponseInterface $response Экземпляр ответа
     *
     * @return array
     */
    protected function prepareResponse(ResponseInterface $response): array
    {
        if ($response->getReasonPhrase() != 'OK') {
            $this->registerErrorResponseEvent($response);
        }
        $responseData = parent::prepareResponse($response);

        if (isset($responseData['result'])) {
            return $responseData['result'];
        }

        $message = reset($responseData['error']);
        $message = is_array($message) ? $message : $responseData['error'];
        $code = (isset($message['code'])) ? $message['code'] : 0;
        $message = $message['message'] ?? implode(' ', $message);

        throw new DomainException($message, $code);
    }

    /**
     * Регистрация события ошибки при получении данных из NG
     *
     * @param ResponseInterface $response Экземпляр ответа
     */
    protected function registerErrorResponseEvent(ResponseInterface $response)
    {
        event(new TransNgErrorResponseEvent($response));
    }
}
